#!/usr/bin/env python3

from jinja2 import Environment, BaseLoader, ChoiceLoader, FunctionLoader, FileSystemLoader
from typing import Any
from urllib.parse import quote as urlquote

from .config import Context, DEFAULT_TEMPLATE_NAME
from .style import STYLESHEET_NAME
from .tree import SourceNode

DEFAULT_TEMPLATE = """
<html>
    <head>
        <title>{{ title }}</title>
        <link rel="stylesheet" href="{{ default_stylesheet }}"/>
    </head>
    <body>
        <h1>{{ title }}</h1>
        {{ text }}
    </body>
</html>
"""


def _create_loader(ctxt: Context) -> BaseLoader:
    return ChoiceLoader([
        FileSystemLoader(ctxt.template_dir),
        FunctionLoader(lambda n: DEFAULT_TEMPLATE if n == DEFAULT_TEMPLATE_NAME else None)
    ])


def create_template_environment(ctxt: Context) -> Environment:
    return Environment(loader=_create_loader(ctxt))


def template_vars(node: SourceNode, ctxt: Context) -> dict[str, Any]:
    tvars = node.make_vars()

    if ctxt.relative_links:
        stylesheet_path = ctxt.source_dir / STYLESHEET_NAME
        rel_path = ctxt.source_dir / node.path
        tvars['default_stylesheet'] = urlquote(stylesheet_path.relative_to(rel_path.parent, walk_up=True).as_posix())
        pass
    else:
        tvars['default_stylesheet'] = urlquote(ctxt.url_prefix + STYLESHEET_NAME)
        pass

    return tvars
